<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 03/25/2004 *
 * ===============                              Last change: 09/29/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : extensions.php                                   *
 * -------------------------------------------------------------------- *
 * Short description : Extension management                             *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Erweiterungen-Management                         *
 * -------------------------------------------------------------------- *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (ereg(basename(__FILE__), $_SERVER['PHP_SELF']))
{
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), "/inc") + 4) . "/security.php";
	require($INC);
}
//
function EXTENSION_REGISTER ($ext_name, $id, $dry_run=false)
{
	global $NOTES, $_CONFIG, $INC_POOL, $cacheInstance;
	// We want to register an extension and registration status is by default "failed" (= false)
	$EXT_LOAD_MODE = "register"; $ret = false; $SQLs = array();
	$INC_POOL = array();

	// This shall never do a non-admin user!
	if (!IS_ADMIN()) return false;

	// Is this extension already installed?
	if (EXT_IS_ACTIVE($ext_name)) return false;

	// Generate file name
	$file = sprintf("%sinc/extensions/ext-%s.php", PATH, $ext_name);

	// Does this extension exists?
	if (file_exists($file) && is_readable($file))
	{
		// Extension was found so we can load it in registration mode
		require($file);

		// And run possible updates
		$EXT_LOAD_MODE = "update"; $EXT_UPDATE_DEPENDS = "";
		foreach ($EXT_VER_HISTORY as $EXT_VER)
		{
			require($file);
			if (((GET_EXT_VERSION("sql_patches") != '') && ($_CONFIG['verbose_sql'] == "Y")) || (!EXT_IS_ACTIVE("sql_patches")))
			{
				if (!empty($UPDATE_NOTES))
				{
					// Update notes found
					$NOTES .= "<STRONG>v".$EXT_VER.":</STRONG><br />".$UPDATE_NOTES."<br /><br />\n";
					$UPDATE_NOTES = "";
				}
				 elseif (($EXT_VER == "0.0") || ($EXT_VER == "0.0.0"))
				{
					// Initial release
					$NOTES .= "<STRONG>v".$EXT_VER.":</STRONG><br />".INITIAL_RELEASE."<br /><br />\n";
				}
				 else
				{
					$NOTES .= "<STRONG>v".$EXT_VER.":</STRONG><br /><I>".NO_UPDATE_NOTES."</I><br /><br />\n";
				}
			}
		}

		// Does this extension depends on an outstanding update of another update?
		if (!empty($EXT_UPDATE_DEPENDS))
		{
			// Backup SQL commands and clear current
			$SQLs2 = $SQLs;  $SQLs = array(); $test = false;

			// Backup language as well
			$LANG_BCK = $EXT_LANG_PREFIX; $EXT_ALWAYS_ACTIVE = "N";

			// Load required extension also in update mode
			$file = sprintf("%sinc/extensions/ext-%s.php", PATH, $EXT_UPDATE_DEPENDS);

			// Check for required file
			if (file_exists($file) && is_readable($file))
			{
				// File exists so let's load it
				$VER_BACKUP = $EXT_VERSION;
				require($file);
				$EXT_VERSION = $VER_BACKUP;

				// If versions mismatch update extension first
				$ext_ver = GET_EXT_VERSION($EXT_UPDATE_DEPENDS);
				if (empty($ext_ver))
				{
					// Extension not registered so far so first load task's ID...
					$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_task_system WHERE task_type='EXTENSION' AND subject LIKE '[%s:]%%' LIMIT 1",
					 array($EXT_UPDATE_DEPENDS), __FILE__, __LINE__);
					if (SQL_NUMROWS($result) == 1)
					{
						// Task found so load task's ID and register extension...
						list($task) = SQL_FETCHROW($result);
						SQL_FREERESULT($result);
						$test = EXTENSION_REGISTER($EXT_UPDATE_DEPENDS, $task, $dry_run);
					}
				}
				 elseif ($ext_ver != $EXT_VERSION)
				{
					// Ok, update this extension now
					EXTENSION_UPDATE(basename($file), $EXT_UPDATE_DEPENDS, $ext_ver, $dry_run);

					// All okay!
					$test = true;
				}
				 else
				{
					// Nothing to register / update before...
					$test = true;
				}
			}
			 else
			{
				// Required file for update does not exists!
				$test = true;
				// But this is fine for the first time...
			}

			// Finally restore previous SQLs
			$SQLs = $SQLs2; unset($SQLs2);
			$EXT_LANG_PREFIX = $LANG_BCK;
		}
		 else
		{
			// Does not depend on an other extension
			$test = true;
		}

		// Switch back to register mode
		$EXT_LOAD_MODE = "register";

		// Does everthing before wents ok?
		if ($test) {
			// "Dry-run-mode" activated?
			if (!$dry_run) {
				// Run all SQLs
				foreach ($SQLs as $sql) {
					// Trim spaces away which we don't need
					$sql = trim($sql);

					// Is there still an SQL query?
					if (!empty($sql)) {
						// Do we have an "ALTER TABLE" command?
						if (substr(strtolower($sql), 0, 11) == "alter table") {
							// Analyse the alteration command
							SQL_ALTER_TABLE($sql, __FILE__, __LINE__);
						} else {
							// Run regular SQL command
							$result = SQL_QUERY($sql, __FILE__, __LINE__, false);
						}
					} // END - if
				} // END - foreach

				// Remove cache file(s) if extension is active
				if ((EXT_IS_ACTIVE("cache")) || (GET_EXT_VERSION("cache") != "")) {
					//* DEBUG: */ echo __LINE__.": DESTROY!<br />\n";
					// Remove cache files
					if ($cacheInstance->cache_file("extensions", true)) $cacheInstance->cache_destroy();
					if ($cacheInstance->cache_file("mod_reg", true))    $cacheInstance->cache_destroy();
					if ($cacheInstance->cache_file("config", true))     $cacheInstance->cache_destroy();
				} // END - if

				// Check for added include files
				if (count($INC_POOL > 0)) {
					// Loads every include file
					foreach ($INC_POOL as $inc) {
						require_once($inc);
					} // END - foreach
				}

				// Register extension
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_extensions (ext_name, ext_lang_file, ext_active, ext_version) VALUES ('%s', '%s', '%s', '%s')",
				 array($ext_name, $EXT_LANG_PREFIX, $EXT_ALWAYS_ACTIVE, $EXT_VERSION), __FILE__, __LINE__);

				// Update task management
				$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_task_system SET status='SOLVED' WHERE id=%d LIMIT 1",
				 array(bigintval($id)), __FILE__, __LINE__);

				// In normal mode return a true on success
				$ret = true; unset($SQLs);
			} else {
				// Rewrite SQL command to keep { and } inside
				foreach ($SQLs as $key => $sql) {
					$sql = str_replace('{', "&#123;", str_replace('}', "&#125;", $sql));
					$SQLs[$key] = $sql;
				} // END - foreach

				// In  "dry-run" mode return array with all SQL commands
				$ret = $SQLs;

				// Remove all SQL commands
				unset($SQLs);
			}
		} else {
			// No, an error occurs while registering extension :-(
			$ret = false;
		}
	} elseif (($id > 0) && (!empty($ext_name))) {
		// Remove task from system when id and extension's name is valid
		$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_task_system WHERE id=%d AND status='NEW' LIMIT 1",
		 array(bigintval($id)), __FILE__, __LINE__);
	}

	// Is this the sql_patches?
	//* DEBUG: */ echo __LINE__.":{$ext_name}/{$EXT_LOAD_MODE}<br />\n";
	if (($ext_name == "sql_patches") && (($EXT_LOAD_MODE == "register") || ($EXT_LOAD_MODE == "remove")) && (!$dry_run) && ($test)) {
		// Then redirect to logout
		//* DEBUG: */ echo __LINE__.": LOAD!<br />\n";
		LOAD_URL("modules.php?module=admin&logout=1&".$EXT_LOAD_MODE."=sql_patches");
	} // END - if

	// Return status code
	return $ret;
}
//
function EXTENSION_RUN_SQLS($id, $EXT_LOAD_MODE) {
	global $cacheInstance;
	$SQLs = array();

	// This shall never do a non-admin user!
	if (!IS_ADMIN()) return false;

	// Get extension's name
	$ext_name = GET_EXT_NAME($id);
	if (empty($ext_name)) return false;

	// Load extension in detected mode
	$file = sprintf("%sinc/extensions/ext-%s.php", PATH, $ext_name);
	if (file_exists($file) && is_readable($file)) require($file);

	if ((is_array($SQLs) && (sizeof($SQLs) > 0))) {
		// Run SQL commands...
		foreach ($SQLs as $sql) {
			// Trim spaces away which we don't need
			$sql = trim($sql);

			// Is there still an SQL query?
			if (!empty($sql)) {
				// Do we have an "ALTER TABLE" command?
				if (substr(strtolower($sql), 0, 11) == "alter table") {
					// Analyse the alteration command
					SQL_ALTER_TABLE($sql, __FILE__, __LINE__);
				} else {
					// Run regular SQL command
					$result = SQL_QUERY($sql, __FILE__, __LINE__, false);
				}
			} // END - if
		} // END - foreach

		// Removal mode?
		if ($EXT_LOAD_MODE == "remove") {
			// Delete this extension (remember to remove it from your server *before* you click on welcome!
			$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_extensions WHERE id=%d LIMIT 1",
			 array($id), __FILE__, __LINE__);
		} // END - if

		// Remove cache file(s) if extension is active
		if ((EXT_IS_ACTIVE("cache")) || (GET_EXT_VERSION("cache") != "")) {
			//* DEBUG: */ echo __LINE__.": DESTROY!<br />\n";
			// Remove cache files
			if ($cacheInstance->cache_file("extensions", true)) $cacheInstance->cache_destroy();
			if ($cacheInstance->cache_file("mod_reg", true))    $cacheInstance->cache_destroy();
			if ($cacheInstance->cache_file("config", true))     $cacheInstance->cache_destroy();
		} // END - if

		// Is this the sql_patches?
		//* DEBUG: */ echo __LINE__.": {$id}/{$ext_name}/{$EXT_LOAD_MODE}<br />\n";
		if (($ext_name == "sql_patches") && (($EXT_LOAD_MODE == "register") || ($EXT_LOAD_MODE == "remove"))) {
			// Then redirect to logout
			//* DEBUG: */ echo __LINE__.": LOAD!<br />\n";
			LOAD_URL("modules.php?module=admin&logout=1&".$EXT_LOAD_MODE."=sql_patches");
		} // END - if
	} // END - if
}
// Check if given extension is active
function EXT_IS_ACTIVE ($ext_name, $ignore_admin = false, $ignore_cache = false) {
	global $cacheArray, $_CONFIG;

	// Extensions are all inactive during installation
	if ((!isBooleanConstantAndTrue('mxchange_installed')) || (isBooleanConstantAndTrue('mxchange_installing')) || (empty($ext_name))) return false;

	// Extension's file name will also be checked
	$file = sprintf("%sinc/extensions/ext-%s.php", PATH, $ext_name);
	if ((!file_exists($file)) && (!is_readable($file))) return false;
	//* DEBUG: */ echo "*".$ext_name."(".count($cacheArray).")<br />";

	// Failed is the default
	$ret = false;
	if ((!empty($cacheArray['extensions']['ext_active'][$ext_name])) && (!$ignore_cache)) {
		// Load from cache
		//* DEBUG: */ echo "CACHE!<br />\n";
		$active = $cacheArray['extensions']['ext_active'][$ext_name];

		// Count cache hits
		if (isset($_CONFIG['cache_hits'])) $_CONFIG['cache_hits']++;
	} else {
		//* DEBUG: */ echo "DB!<br />\n";
		// Load from database
		$result = SQL_QUERY_ESC("SELECT ext_active FROM "._MYSQL_PREFIX."_extensions WHERE ext_name='%s' LIMIT 1",
		 array($ext_name), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 0) {
			// Extension was not found!
			return false;
		}
		list($active) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);

		// Write cache array
		$cacheArray['extensions']['ext_active'][$ext_name] = $active;
	}

	// Is this extension activated? (For admins we always have active extensions...)
	$inc = sprintf("%sinc/extensions/ext-%s.php", PATH, $ext_name);
	// Shorter way
	return (
		(
			($active == "Y") || (
				(IS_ADMIN()) &&
				(!$ignore_admin) &&
				(!empty($active))
			)
		) && (
			file_exists($inc)
		) && (
			is_readable($inc)
		)
	);
}
// Get version from extensions
function GET_EXT_VERSION ($ext_name) {
	global $cacheArray, $_CONFIG, $cacheInstance;
	$ret = false;

	// Extensions are all inactive during installation
	if ((!isBooleanConstantAndTrue('mxchange_installed')) || (isBooleanConstantAndTrue('mxchange_installing'))) return "";

	// Is the cache written?
	if (!empty($cacheArray['extensions']['ext_version'][$ext_name])) {
		// Load data from cache
		$ret = $cacheArray['extensions']['ext_version'][$ext_name];

		// Count cache hits
		if (isset($_CONFIG['cache_hits'])) $_CONFIG['cache_hits']++; else $_CONFIG['cache_hits'] = 1;
	} elseif (!is_object($cacheInstance)) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT ext_version FROM "._MYSQL_PREFIX."_extensions WHERE ext_name='%s' LIMIT 1",
		 array($ext_name), __FILE__, __LINE__);
		list($ret) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);

		// Set cache
		$cacheArray['extensions']['ext_version'][$ext_name] = $ret;
	}
	return $ret;
}
//
function EXTENSION_UPDATE($file, $ext, $EXT_VER, $dry_run=false)
{
	// This shall never do a non-admin user!
	global $cacheInstance, $_CONFIG, $NOTES; $SQLs = array();
	if ((!IS_ADMIN()) || (empty($ext))) return false;

	// Load extension in update mode
	$EXT_LOAD_MODE = "update"; $EXT_UPDATE_DEPENDS = ""; $NOTES = "";
	include(PATH."inc/extensions/".$file);
	if (!empty($EXT_UPDATE_DEPENDS))
	{
		// Update another extension first!
		$test = EXTENSION_UPDATE(("ext-".$EXT_UPDATE_DEPENDS.".php"), $EXT_UPDATE_DEPENDS, GET_EXT_VERSION($EXT_UPDATE_DEPENDS), $dry_run);
	}

	// Check if version is updated
	if ((($EXT_VERSION != $EXT_VER) || ($dry_run)) && (is_array($EXT_VER_HISTORY)))
	{
		// Search for starting point
		$start = array_search($EXT_VER, $EXT_VER_HISTORY);
		$NOTES = "";

		// And load SQL queries in order of version history
		for ($idx = ($start + 1); $idx < sizeof($EXT_VER_HISTORY); $idx++)
		{
			// Remove old SQLs array to prevent possible bugs
			if (!$dry_run) { unset($SQLs); $SQLs = array(); }

			// Set version
			$EXT_VER = $EXT_VER_HISTORY[$idx];

			// Include again...
			include(PATH."inc/extensions/".$file);

			// Add notes
			if ($_CONFIG['verbose_sql'] == "Y")
			{
				$EXT_VER = $EXT_VER_HISTORY[$idx];
				if (!empty($UPDATE_NOTES))
				{
					// Update notes found
					$NOTES .= "<STRONG>v".$EXT_VER.":</STRONG><br />".$UPDATE_NOTES."<br /><br />\n";
					$UPDATE_NOTES = "";
				}
				 elseif ($EXT_VER == "0.0")
				{
					// Initial release
					$NOTES .= "<STRONG>v".$EXT_VER.":</STRONG><br />".INITIAL_RELEASE."<br /><br />\n";
				}
				 else
				{
					$NOTES .= "<STRONG>v".$EXT_VER.":</STRONG><br /><I>".NO_UPDATE_NOTES."</I><br /><br />\n";
				}
			}

			// Run SQLs
			if ((is_array($SQLs)) && (!$dry_run))
			{
				// Run SQL commands
				foreach ($SQLs as $sql)
				{
					$sql = trim($sql);
					if (!empty($sql))
					{
						// Do we have an "ALTER TABLE" command?
						if (substr(strtolower($sql), 0, 11) == "alter table") {
							// Analyse the alteration command
							SQL_ALTER_TABLE($sql, __FILE__, __LINE__);
						} else {
							// Run regular SQL command
							$result = SQL_QUERY($sql, __FILE__, __LINE__, false);
						}
					}
				}
			} elseif (GET_EXT_VERSION("sql_patches") == "") {
				// Remove SQLs if extension is not installed
				$SQLs = array();
			}
		}

		if (!$dry_run)
		{
			// In normal mode insert task and update extension's version...
			$ext_subj = "[UPDATE-".$ext."-".$EXT_VERSION.":] ".ADMIN_UPDATE_EXT_SUBJ;

			// Create task
			$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_task_system WHERE subject='%s' LIMIT 1",
			 array($ext_subj), __FILE__, __LINE__);
			if (SQL_NUMROWS($result) == 0)
			{
				// Task not created so it's a brand-new extension which we need to register and create a task for!
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_task_system (assigned_admin, userid, status, task_type, subject, text, task_created) VALUES ('%s', '0', 'NEW', 'EXTENSION_UPDATE', '%s', '%s', UNIX_TIMESTAMP())",
				 array(GET_ADMIN_ID(SQL_ESCAPE(get_session('admin_login'))), $ext_subj, addslashes($NOTES)), __FILE__, __LINE__);
			}

			// Free memory
			SQL_FREERESULT($result);

			// Update extension's version
			$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_extensions SET ext_version='%s' WHERE ext_name='%s' LIMIT 1",
			 array($EXT_VERSION, $ext), __FILE__, __LINE__);

			// Update cache
			if (EXT_IS_ACTIVE("cache"))
			{
				if ($cacheInstance->cache_file("extensions", true) == true) $cacheInstance->cache_destroy();
				if ($cacheInstance->cache_file("config", true) == true)     $cacheInstance->cache_destroy();
				if ($cacheInstance->cache_file("mod_reg", true) == true)    $cacheInstance->cache_destroy();
			}

			// Remove array
			unset($SQLs);
		}
		 else
		{
			// In "dry-run" mode return array with SQL commands
			return $SQLs;
		}
	}
}
//
function EXTENSION_VERBOSE_TABLE($SQLs, $title = ADMIN_SQLS_EXECUTED_ON_REMOVAL, $dashed = "", $switch = false, $WIDTH = "480") {
	global $_CONFIG;

	$S = false; $SW = 2; $i = 1;
	$OUT = "";
	if ((is_array($SQLs)) && (GET_EXT_VERSION("sql_patches")) && ($_CONFIG['verbose_sql'] == "Y")) {
		$OUT  = "<DIV align=\"center\">
<TABLE border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"".$WIDTH."\" align=\"center\"".$dashed.">
<TR>
  <TD colspan=\"2\" align=\"center\" class=\"admin_title bottom2\" height=\"24\">
    <STRONG>".$title.":</STRONG>
  </TD>
</TR>\n";
		foreach ($SQLs as $idx => $sql) {
			$sql = trim($sql);
			if (!empty($sql)) {
				$S = true;
				$OUT .= "<TR>
  <TD class=\"switch_sw".(3 - $SW)." bottom2\" style=\"padding-left: 2px\" width=\"30\">".$i.".</TD>
  <TD class=\"switch_sw".(3 - $SW)." bottom2\" style=\"padding-left: 5px; padding-right: 5px\">
    ".$sql."
  </TD>
</TR>\n";
				if ($switch) $SW = 3 - $SW;
				$i++;
			}
		}
	}

	if ((!$S) && (GET_EXT_VERSION("sql_patches")) && ($_CONFIG['verbose_sql'] == "Y")) {
		// No addional SQL commands to run
		$OUT .= "<TR>
  <TD colspan=\"2\" align=\"center\" class=\"switch_sw2 bottom2\" height=\"24\">
    <FONT class=\"admin_note\">".ADMIN_NO_ADDIONAL_SQLS."</FONT>
  </TD>
</TR>\n";
	}

	if (!empty($OUT)) {
		// Add missing close-table tag
		$OUT .= "</TABLE>
</DIV>\n";
	}

	// Return output
	return $OUT;
}
//
function GET_EXT_NAME($id)
{
	$ret = "";
	global $cacheArray, $_CONFIG;
	if (!empty($cacheArray['extensions']['ext_id'][$id]))
	{
		// Load from cache
		$ret = $cacheArray['extensions']['ext_id'][$id];

		// Count cache hits
		$_CONFIG['cache_hits']++;
	}
	 else
	{
		// Load from database
		$result = SQL_QUERY_ESC("SELECT ext_name FROM "._MYSQL_PREFIX."_extensions WHERE id=%d LIMIT 1",
		 array(bigintval($id)), __FILE__, __LINE__);
		list($ret) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);
	}
	return $ret;
}
//
function GET_EXT_ID($name)
{
	$ret = "0";
	global $cacheArray, $_CONFIG;
	if ((isset($cacheArray['extensions']['ext_id'])) && (is_array($cacheArray['extensions']['ext_id'])))
	{
		// Load from cache
		$ret = array_search($name, $cacheArray['extensions']['ext_id']);

		// Count cache hits
		$_CONFIG['cache_hits']++;
	}
	 else
	{
		// Load from database
		$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_extensions WHERE ext_name='%s' LIMIT 1",
		 array($name), __FILE__, __LINE__);
		list($ret) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);
	}
	return $ret;
}
//
?>
